!     Function to return off-diagonal elements of potential in atomic units.
!     ista and istb give the "fixed" indices of the two surfaces 
!     (that is, their indices as defined by the neps value in the RASTER program).
!     Return value should be symmetric to interchange of ista and istb. 

!  This version uses couplings between B and A 
!     and B and R implemented as splines.
!     The A/B coupling is multiplied by abmagfac (defined in coupling.fi)
!     to produce more A-state trajectories for analysis.
!     The B-R coupling has an additional empirical term added at long range.

!     For A-X and A-R couplings, this version uses a function that falls
!     off as a Lorentzian in delta-V.  


      double precision function couple(rb, rs, gamma, ista,istb,va,vb)
      implicit none
      double precision rb, rs, gamma, va, vb
      integer ista, istb

      integer istlow, isthigh
      double precision r1, r2, r3, alpha, arg, ro2
      double precision rlong, rshort, rmid, deltav
      double precision vBA_func, vRB_func
      double precision auperev, pi
      parameter(auperev = 1.0d0/27.2114d0) 

      include 'coupling.fi'  ! definitions of arconst, axconst, etc.
      pi = dacos(-1.0d0)
      istlow = min(ista, istb)
      isthigh = max(ista, istb)

      if ((istlow.eq.1).and.(isthigh.eq.4))   then  ! B-A coupling
         ! get valence coordinates
         call trans1(rb,rs,gamma,r1,r2,alpha)
         alpha = 180*alpha/pi

         couple = abmagfac*vBA_func(r1,r2,alpha)*auperev

      else if ((istlow.eq.1).and.(isthigh.eq.2))   then  ! A-X coupling
!  this version just uses plain Lorentzian
         deltav = vb-va
         couple = axconst*arxwidth**2/(deltav**2+arxwidth**2)

! commented-out version restricts Lorentzian to specific range in angle
c$$$         ! get three distances
c$$$         call trans2(rb,rs,gamma,r1,r2,r3)
c$$$         rlong = max(r1, r2, r3)
c$$$         rshort = min(r1,r2,r3)
c$$$         rmid = r1+r2+r3-rlong-rshort
c$$$         ! find angle opposite longest side
c$$$         arg = (rshort**2+rmid**2-rlong**2)/(2*rshort*rmid)
c$$$         if (dabs(arg).gt.1.0d0) then
c$$$            arg = sign(1.0d0, arg)
c$$$         end if
c$$$         alpha = acos(arg)
c$$$         alpha = 180*alpha/pi
c$$$         ! use hyperbolic coupling within angular range
c$$$         if ((alpha.ge.60).and.(alpha.le.85)) then
c$$$            deltav = vb-va
c$$$            couple = axconst*arxwidth**2/(deltav**2+arxwidth**2)
c$$$         else
c$$$            couple = 0.0d0
c$$$         end if


      else if ((istlow.eq.1).and.(isthigh.eq.3)) then  ! A-R coupling
         deltav = vb-va
         if (abs(deltav).lt.1.0d0*auperev) then
            couple = arconst*arxwidth**2/(deltav**2+arxwidth**2)
         else
            couple = 0.0d0
         end if

      else if ((istlow.eq.3).and.(isthigh.eq.4)) then ! B-R coupling 
         ! get valence coordinates
         call trans1(rb,rs,gamma,r1,r2,alpha)
         alpha = 180*alpha/pi
         couple = vRB_func(r1,r2,alpha)*auperev

c         ! add on long-range piece
c         r3 = rs
c         ro2 = min(r1, r2, r3)
c         if (ro2.gt.brlrs) then
c            couple = couple + 
c     1           brlramp*(ro2-brlrs)**2*exp(-(ro2-brlrs)/brlrdec)
c         end if

      else  !no other couplings installed
         couple = 0.0d0
      end if

      return
      end

! set up coupling functions at beginning of run.  Notice that
! the printed-out messages about what couplings are used are
! only partially coupled to the code above; if you change the 
! stuff in couple(), it's your job to change the messages here too
! if you want them to mean something.
      subroutine couple_init
      implicit none
      integer iunit(2), neps
      double precision auperev
      parameter(auperev = 1.0d0/27.2114d0) 

      include 'coupling.fi'

      !  set up A-B coupling spline
      iunit(1) = 51
      iunit(2) = 52

      do neps=1,2
         call raster_BA(iunit(neps), neps)
         write(*,'(A,I2)') 'raster_BA done: coupling data on unit ', 
     1        iunit(neps)
      end do
         write(*,*) 'Note: A/B coupling magnified by factor of ',
     1     abmagfac, '!'                   !'

      !  set up B-R coupling spline
      iunit(1) = 41
      iunit(2) = 42
      do neps=1,2
         call raster_RB(iunit(neps), neps)
         write(*,'(A,I2)') 'raster_RB done: coupling data on unit ', 
     1        iunit(neps)
      end do

! Write description into output of coupling functions
      write(*,*) 'Lorentzian in deltaV coupling for A-R and A-X'
      write(*,*) 'arxwidth = ', arxwidth/auperev, ' eV'
      write(*,*) 'axconst = ', axconst/auperev, ' eV'
      write(*,*) 'arconst = ', arconst/auperev, ' eV'
      write(*,*) 'A-R restricted to |deltaV|<1 eV'
c     write(*,*) 'Additional long-range piece added to B-R coupling'
c     write(*,*) 'of form a*r**2*exp(-r/dec) where r = (ro2-s)'
c     write(*,*) 'a, s, dec = ',brlramp, brlrs, brlrdec

      return
      end

