! Demonstration program for the O3 diabatic potential programs of
!     Reinhard Schinke and George McBane, JCP 132, 044305 (2010)
! This file shows how to use the functions that set up and evaluate the couplings
! between the different potential surfaces.
! See splinetest.f for evaluating the potential energies themselves.
! 29 April 2014 G. McBane.

!     Potentials prepared by RS; evaluation code is natural cubic
!     spline, based on old code by Claude Leforestier and used for many
!     years by RS, with the present interfaces mostly written by GCM.

! Main potential data is contained in unformatted files, one per
! electronic state, which (for most Fortran compilers) must be named (or
! linked to the names) fort.31, fort.32, fort.33, and fort.34.  As
! supplied these give data for states A, X, R, B in that order.

! Coupling data for the BR and BA couplings are similarly stored as splines,
! in files fort.41, 42, 51, 52.  These are read in during the call to couple_init().

! To run this test, you need to compile and link this file, potutil.f,
! rasterunf.f, and naturl2.f, couple2.f, vba.f, reinhardvRB.f, and trans1.f,
! and you need to also link the standard
! BLAS routine dgemm.  (You can link DGEMM by linking any standard
! high-performance library such as MKL, ATLAS, or ACML, or by
! downloading 'dgemm.f plus dependencies' from the BLAS section of
! netlib.org and compiling and linking that along with the other files.)
! The include files coupling.fi, surfaces.fi and splinedata.fi need to be in the
! directory with the .f files when you compile.  Then the spline data
! files fort.31...fort.34, fort.41, fort.42, fort.51, fort.52  need to be in 
! the working directory when you
! run the executable program. The coupling programs assume that the executable program will
! search for the file fort.41 when it is asked to read an unopened file on unit 41, and 
! so on.  Most modern Fortran compilers produce programs that do that.

! Please send questions to George McBane, mcbaneg@gvsu.edu
! 29 April 2014

      program cpltest
      implicit double precision (A-H,O-Z)
      integer iunit(4)
      data iunit/31,32,33,34/  ! fortran unit numbers; files fort.31, fort.32, etc
      character*7 filenames(4)
      data filenames /'fort.31','fort.32','fort.33','fort.34'/
      include 'surfaces.fi'  ! defines nsurf=4
      double precision v(nsurf)
      double precision auperev
      parameter(auperev = 3.674932540d-2) !Hartree per eV
      
      common/state/istate ! communicates desired electronic state to pot3 routine

! read in spline data files and construct necessary data arrays
      do 30 neps=1,nsurf  ! loop over four surfaces
         open(iunit(neps),file=filenames(neps),form='unformatted')
         rewind iunit(neps)
         call rasterunf(iunit(neps),neps) ! read and store spline coefficients for state neps 
         close (iunit(neps))
         write(6,31) surfnames(neps), iunit(neps)
 31      format(/,'  raster done: surface ',A1, ' on unit ', I3)
 30   continue

      call couple_init          ! initialize coupling functions, read spline data for coupling

      r1 = 3.25d0  ! valence coordinates, following fig. 8 in paper
      r2 = 2.7d0
      alpha = 120.0d0

! convert from valence coords to symmetric jacobi coords
      rs=dsqrt(r1**2+r2**2-2*r1*r2*dcos(alpha/57.29577951d0))
      rb=dsqrt(r1**2+r2**2-2*(rs/2.)**2)/dsqrt(2.d0)
      cos_gamma=-(r1**2-rb**2-(rs/2.)**2)/(rb*rs)
      gamma=dacos(cos_gamma)


      ista = 3  !choose states; order is A, X, R, B, so 3 and 4 gives R-B coupling
      istb = 4

      istate = ista     ! pot3 can see istate
      va = pot3(rb, rs, gamma)  ! get potential for state ista
      istate = istb    
      vb = pot3(rb, rs, gamma) ! get potential for state istb
      
      v12 = couple(rb, rs, gamma, ista,istb,va,vb)  ! get coupling potential
      write(*,*) 'ista = ', ista, 'istb = ', istb
      write(*,*) 'v1 = ', va/auperev, ' v2 = ', vb/auperev
      write(*,*) 'v12 = ', v12/auperev
      


      end



      
